import socket
import keyboard
import time

from threading import Thread


class LiteNetClient:
    def __init__(self, ip, port=5050, header=64, encoding="utf-8", debug=False):
        self.ip, self.port = ip, port

        self.header, self.encoding = header, encoding

        self._close_msg = "[CLOSE]"

        socket.setdefaulttimeout(5)

        keyboard.add_hotkey("t", self.message)

        self.debug = debug

        self._temp = None

    def start(self):
        Thread(target=self._start).start()

    def message(self):
        message = input("You >> ")
        self.send(message)

    def get_server_msg(self):
        try:
            msg_length = self.socket.recv(self.header).decode(self.encoding)

            if msg_length:
                msg = self.socket.recv(int(msg_length)).decode(self.encoding)

                return msg

            else:
                return None

        except socket.timeout:
            return None

        except socket.error:
            print("Server has closed connection your with the server.")
            time.sleep(2)
            exit()

    def send(self, msg):
        msg = msg.encode(self.encoding)
        msg_length = len(msg)
        send_length = str(msg_length).encode(self.encoding) + b' ' * (self.header - len(str(msg_length).encode(self.encoding)))
        self.socket.send(send_length)
        self.socket.send(msg)
        if self.debug:
            print("[DEBUG] Message Sent")

    def _start(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        self.socket.connect((self.ip, self.port))
        print(f"Connected to {self.ip}:{self.port}")

        while True:
            server_msg = self.get_server_msg()
            if server_msg:
                print(server_msg)

    def disconnect(self):
        self.send(self._close_msg)
        self.socket.close()



import socket
import threading

from litenet.utils import to_liteaddr


class LiteNetServer:
    def __init__(self, ip, port=5050, header=64, encoding="utf-8", debug=False):
        self.ip, self.port = ip, port

        self.header, self.encoding = header, encoding

        self._close_msg = "[CLOSE]"

        self.clients = {}

        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        self.debug = debug

        self._stopped = False

    def start(self):
        threading.Thread(target=self._start).start()

    def _start(self):
        self.server.bind((self.ip, self.port))
        self.server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server.listen()

        print(f"LiteNet Server Listening on {self.ip}:{self.port}")

        while not self._stopped:
            conn, addr = self.server.accept()
            thread = threading.Thread(target=self.handle_client, args=(conn, addr))
            thread.start()
            print(f"[NEW CONNECTION] {addr}")

            if self.debug:
                print(self.active_client_count)
                print(self.clients, "\n")
        self.server.close()

    def handle_client(self, conn, addr):
        self.clients[addr] = {
            "alias": to_liteaddr(addr),
            "connection": conn,
            "alive": True
        }

        while self.clients[addr]["alive"]:
            msg_length = self.clients[addr]["connection"].recv(self.header)
            if msg_length:
                msg_length = int(msg_length.decode(self.encoding))

                msg = self.clients[addr]["connection"].recv(msg_length).decode(self.encoding)

                if msg == self._close_msg:
                    break

                print(addr[0] + ":" + str(addr[1]), ">>", msg)

                msg = str(f"{to_liteaddr(addr[0]) + ':' + str(addr[1])} >> {msg}")

                for client in self.clients.keys():
                    if client != addr:
                        self.clients.get(client)["connection"].send(
                            msg.encode(self.encoding) + b" " * (self.header - len(msg.encode(self.encoding)))
                        )

                        self.clients.get(client)["connection"].send(
                            msg.encode(self.encoding)
                        )

        conn.close()
        self.close_client(addr)

    def close_client(self, addr):
        if addr in self.clients:
            self.clients[addr]["alive"] = False
        else:
            raise socket.error(f"{addr} isn't connected to the server.")

    @property
    def active_client_count(self):
        return threading.activeCount()

    def stop(self):
        self._stopped = True



import click
from litenet.server import LiteNetServer
from litenet.client import LiteNetClient


@click.group()
def cli():
    pass



@click.command(help="Launch a LiteNet Server")
@click.option("-h", "--host", required=True, type=str, help="The ip to run the server on. (Must have access to it.)")
@click.option("-p", "--port", default=5050, type=int, help="The port to run the server on. (Must be a valid port.)")
@click.option("--header", "header", default=64, type=int, help="The size of the server message header.")
def server(host, port, header):
    litenet = LiteNetServer(host, port, header=header)
    litenet.start()


@click.command(help="Launch the LiteNet Client")
@click.option("-h", "--host", required=True, type=str, help="The ip the server's running on. (Must have access to it.)")
@click.option("-p", "--port", default=5050, type=int, help="The port the server's running on. (Must be a valid port.)")
@click.option("--header", "header", default=64, type=int, help="The size of the server message header. (Must match the server's.)")
def client(host, port, header):
    client = LiteNetClient(host, port, header)
    client.start()


cli.add_command(server)
cli.add_command(client)

if __name__ == '__main__':
    cli()



def to_liteaddr(ip):
    ip = ip[0].split(".")
    return "Lite|" + "_".join([int_to_lite_num(int(n)) for n in ip])


def int_to_lite_num(number):
    row = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

    base = len(row)

    lite_num = [(number - (number % base)) / base, number % base]

    return "".join([list(row)[int(digit) - 1] for digit in lite_num])



from setuptools import setup

setup(
    name='litenet',
    version='0.2.3.2',
    packages=['litenet'],
    url='https://github.com/GrandMoff100/LiteNet',
    license='',
    author='Quantum_Wizard',
    author_email='',
    description='A CL version of LiteNet',
    install_requires=["click", "keyboard"],
    entry_points={
        "console_scripts": ["litenet=litenet.cli:cli"]
    }
)
